@@ -3,26 +3,29 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2024 Zuse Institute Berlin (ZIB)                       */
+/* Copyright (C) 2020-2025 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software: you can redistribute it and/or modify      */
-/* it under the terms of the GNU Lesser General Public License as published  */
-/* by the Free Software Foundation, either version 3 of the License, or      */
-/* (at your option) any later version.                                       */
+/* Licensed under the Apache License, Version 2.0 (the "License");           */
+/* you may not use this file except in compliance with the License.          */
+/* You may obtain a copy of the License at                                   */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*     http://www.apache.org/licenses/LICENSE-2.0                            */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program.  If not, see <https://www.gnu.org/licenses/>.    */
+/* Unless required by applicable law or agreed to in writing, software       */
+/* distributed under the License is distributed on an "AS IS" BASIS,         */
+/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  */
+/* See the License for the specific language governing permissions and       */
+/* limitations under the License.                                            */
+/*                                                                           */
+/* You should have received a copy of the Apache-2.0 license                 */
+/* along with PaPILO; see the file LICENSE. If not visit scipopt.org.        */
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifndef _PAPILO_PRESOLVERS_DOMINATED_COLS_HPP_
 #define _PAPILO_PRESOLVERS_DOMINATED_COLS_HPP_
 
+#include "papilo/Config.hpp"
 #include "papilo/core/PresolveMethod.hpp"
 #include "papilo/core/Problem.hpp"
 #include "papilo/core/ProblemUpdate.hpp"
@@ -149,9 +152,9 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
    // parallel
 #ifdef PAPILO_TBB
    tbb::parallel_for(
-       tbb::blocked_range<int>( 0, ncols ),
-       [&]( const tbb::blocked_range<int>& r ) {
-          for( int col = r.begin(); col < r.end(); ++col )
+       tbb::blocked_range<unsigned int>( 0, ncols ),
+       [&]( const tbb::blocked_range<unsigned int>& r ) {
+          for( unsigned int col = r.begin(); col < r.end(); ++col )
 #else
    for( int col = 0; col < (int)ncols; ++col )
 #endif
@@ -390,8 +393,8 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
    while( ndomcols < ndomcolsbound && start < unboundedcols.size() )
    {
       int ndomcolsbuffers = 0;
-      size_t base = start;
-      size_t stopp;
+      unsigned int base = start;
+      unsigned int stopp;
 
       // find dominations until number of columns is reached
       while( ndomcols < ncols && start < unboundedcols.size() )
@@ -405,9 +408,9 @@ DominatedCols<REAL>::execute( const Problem<REAL>& problem,
 #ifdef PAPILO_TBB
    // scan unbounded columns if they dominate other columns
    tbb::parallel_for(
-       tbb::blocked_range<int>( start, stopp ),
-       [&]( const tbb::blocked_range<int>& r ) {
-          for( int k = r.begin(); k < r.end(); ++k )
+       tbb::blocked_range<unsigned int>( start, stopp ),
+       [&]( const tbb::blocked_range<unsigned int>& r ) {
+          for( unsigned int k = r.begin(); k < r.end(); ++k )
 #else
    for( int k = start; k < (int)stopp; ++k )
 #endif
