@@ -3,27 +3,29 @@
 /*               This file is part of the program and library                */
 /*    PaPILO --- Parallel Presolve for Integer and Linear Optimization       */
 /*                                                                           */
-/* Copyright (C) 2020-2023 Konrad-Zuse-Zentrum                               */
-/*                     fuer Informationstechnik Berlin                       */
+/* Copyright (C) 2020-2025 Zuse Institute Berlin (ZIB)                       */
 /*                                                                           */
-/* This program is free software: you can redistribute it and/or modify      */
-/* it under the terms of the GNU Lesser General Public License as published  */
-/* by the Free Software Foundation, either version 3 of the License, or      */
-/* (at your option) any later version.                                       */
+/* Licensed under the Apache License, Version 2.0 (the "License");           */
+/* you may not use this file except in compliance with the License.          */
+/* You may obtain a copy of the License at                                   */
 /*                                                                           */
-/* This program is distributed in the hope that it will be useful,           */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
-/* GNU Lesser General Public License for more details.                       */
+/*     http://www.apache.org/licenses/LICENSE-2.0                            */
 /*                                                                           */
-/* You should have received a copy of the GNU Lesser General Public License  */
-/* along with this program.  If not, see <https://www.gnu.org/licenses/>.    */
+/* Unless required by applicable law or agreed to in writing, software       */
+/* distributed under the License is distributed on an "AS IS" BASIS,         */
+/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  */
+/* See the License for the specific language governing permissions and       */
+/* limitations under the License.                                            */
+/*                                                                           */
+/* You should have received a copy of the Apache-2.0 license                 */
+/* along with PaPILO; see the file LICENSE. If not visit scipopt.org.        */
 /*                                                                           */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifndef _PAPILO_PRESOLVERS_DUAL_FIX_HPP_
 #define _PAPILO_PRESOLVERS_DUAL_FIX_HPP_
 
+#include "papilo/Config.hpp"
 #include "papilo/core/PresolveMethod.hpp"
 #include "papilo/core/Problem.hpp"
 #include "papilo/core/ProblemUpdate.hpp"
@@ -41,6 +43,7 @@ class DualFix : public PresolveMethod<REAL>
    DualFix() : PresolveMethod<REAL>()
    {
       this->setName( "dualfix" );
+      this->setArgument( ArgumentType::kDual );
       this->setTiming( PresolverTiming::kMedium );
    }
 
@@ -70,8 +73,10 @@ class DualFix : public PresolveMethod<REAL>
 
    PresolveStatus
    execute( const Problem<REAL>& problem,
-            const ProblemUpdate<REAL>& problemUpdate, const Num<REAL>& num,
-            Reductions<REAL>& reductions, const Timer& timer ) override;
+            const ProblemUpdate<REAL>& problemUpdate,
+            const Num<REAL>& num, Reductions<REAL>& reductions,
+            const Timer& timer, int& reason_of_infeasibility)
+       override;
 
  private:
    PresolveStatus
@@ -97,8 +102,8 @@ template <typename REAL>
 PresolveStatus
 DualFix<REAL>::execute( const Problem<REAL>& problem,
                         const ProblemUpdate<REAL>& problemUpdate,
-                        const Num<REAL>& num, Reductions<REAL>& reductions, const Timer& timer )
-{
+                        const Num<REAL>& num, Reductions<REAL>& reductions,
+                        const Timer& timer, int& reason_of_infeasibility){
    const auto& consMatrix = problem.getConstraintMatrix();
    const Vec<RowActivity<REAL>>& activities = problem.getRowActivities();
    const Vec<ColFlags>& cflags = problem.getColFlags();
@@ -133,6 +138,8 @@ DualFix<REAL>::execute( const Problem<REAL>& problem,
    {
       for( int col = 0; col < ncols; ++col )
       {
+         if( reductions.size() >= problemUpdate.getPresolveOptions().max_reduction_seq )
+            break;
          PresolveStatus local_status = perform_dual_fix_step(
              num, reductions, consMatrix, activities, cflags, objective, lbs,
              ubs, rflags, lhs, rhs, col, noStrongReductions, skip_variable_tightening, bound_tightening_offset );
@@ -267,6 +274,8 @@ DualFix<REAL>::perform_dual_fix_step(
       {
          TransactionGuard<REAL> guard{ reductions };
          reductions.lockCol( i );
+         for( int row = 0; row < collen; ++row )
+            reductions.lockRow( rowinds[row] );
          reductions.fixColNegativeInfinity( i, collen, rowinds );
          return PresolveStatus::kReduced;
       }
@@ -278,10 +287,10 @@ DualFix<REAL>::perform_dual_fix_step(
       if( !cflags[i].test( ColFlag::kUbInf ) )
       {
          TransactionGuard<REAL> guard{ reductions };
-
          reductions.lockCol( i );
+         for( int row = 0; row < collen; ++row )
+            reductions.lockRow( rowinds[row] );
          reductions.fixCol( i, ubs[i] );
-
          return PresolveStatus::kReduced;
       }
       else if( objective[i] != 0 )
@@ -292,6 +301,8 @@ DualFix<REAL>::perform_dual_fix_step(
       {
          TransactionGuard<REAL> guard{ reductions };
          reductions.lockCol( i );
+         for( int row = 0; row < collen; row++)
+            reductions.lockRow( rowinds[row] );
          reductions.fixColPositiveInfinity( i, collen, rowinds );
          return PresolveStatus::kReduced;
       }
@@ -338,7 +349,7 @@ DualFix<REAL>::perform_dual_fix_step(
          // if possible, but increase the bound slightly
          if( skip_variable_tightening  && ! cflags[i].test( ColFlag::kUbInf ) )
             return PresolveStatus::kUnchanged;
-         bool skip_performing = false;
+         bool skip = false;
          bool new_ub_init = false;
          REAL new_ub = REAL(0);
          int best_row = -1;
@@ -360,7 +371,7 @@ DualFix<REAL>::perform_dual_fix_step(
                {
                   check_row( activities[row].ninfmax, activities[row].max,
                              rhs[row], values[j], lbs[i],
-                             cflags[i].test( ColFlag::kLbInf ), skip_performing,
+                             cflags[i].test( ColFlag::kLbInf ), skip,
                              cand_bound );
                }
                else
@@ -373,7 +384,7 @@ DualFix<REAL>::perform_dual_fix_step(
                {
                   check_row( activities[row].ninfmin, activities[row].min,
                              lhs[row], values[j], lbs[i],
-                             cflags[i].test( ColFlag::kLbInf ), skip_performing,
+                             cflags[i].test( ColFlag::kLbInf ), skip,
                              cand_bound );
                }
                else
@@ -381,7 +392,7 @@ DualFix<REAL>::perform_dual_fix_step(
                   continue;
             }
 
-            if( skip_performing )
+            if( skip )
                break;
 
             // Only if variable is greater than or equal to new_UB, all rows
@@ -399,17 +410,16 @@ DualFix<REAL>::perform_dual_fix_step(
                // check if bound is already equal or worse than current bound
                // and abort in that case
                if( ( !cflags[i].test( ColFlag::kUbInf ) &&
-                     num.isGE( new_ub, ubs[i] ) ) ||
-                   new_ub >= num.getHugeVal() )
+                     num.isGE( new_ub, ubs[i] ) ) || num.isHugeVal( new_ub ) )
                {
-                  skip_performing = true;
+                  skip = true;
                   break;
                }
             }
          }
 
          // set new upper bound
-         if( !skip_performing && new_ub_init && !num.isHugeVal( new_ub ) )
+         if( !skip && new_ub_init && !num.isHugeVal( new_ub ) )
          {
             assert( cflags[i].test( ColFlag::kUbInf ) || new_ub < ubs[i] );
 
@@ -453,7 +463,7 @@ DualFix<REAL>::perform_dual_fix_step(
          // if possible, but increase the bound slightly
          if( skip_variable_tightening  && ! cflags[i].test( ColFlag::kLbInf ) )
             return PresolveStatus::kUnchanged;
-         bool skip_ = false;
+         bool skip = false;
          bool new_lb_init = false;
          REAL new_lb = REAL(0);
          int best_row = -1;
@@ -475,7 +485,7 @@ DualFix<REAL>::perform_dual_fix_step(
                {
                   check_row( activities[row].ninfmax, activities[row].max,
                              rhs[row], values[j], ubs[i],
-                             cflags[i].test( ColFlag::kUbInf ), skip_,
+                             cflags[i].test( ColFlag::kUbInf ), skip,
                              cand_bound );
                }
                else
@@ -488,7 +498,7 @@ DualFix<REAL>::perform_dual_fix_step(
                {
                   check_row( activities[row].ninfmin, activities[row].min,
                              lhs[row], values[j], ubs[i],
-                             cflags[i].test( ColFlag::kUbInf ), skip_,
+                             cflags[i].test( ColFlag::kUbInf ), skip,
                              cand_bound );
                }
                else
@@ -496,7 +506,7 @@ DualFix<REAL>::perform_dual_fix_step(
                   continue;
             }
 
-            if( skip_ )
+            if( skip )
                break;
 
             // Only if variable is less than or equal to new_LB, all rows in
@@ -513,17 +523,16 @@ DualFix<REAL>::perform_dual_fix_step(
                // check if bound is already equal or worse than current bound
                // and abort in that case
                if( ( !cflags[i].test( ColFlag::kLbInf ) &&
-                     num.isLE( new_lb, lbs[i] ) ) ||
-                   new_lb <= -num.getHugeVal() )
+                     num.isLE( new_lb, lbs[i] ) ) || num.isHugeVal( new_lb ) )
                {
-                  skip_ = true;
+                  skip = true;
                   break;
                }
             }
          }
 
          // set new lower bound
-         if( !skip_ && new_lb_init && !num.isHugeVal( new_lb ) )
+         if( !skip && new_lb_init && !num.isHugeVal( new_lb ) )
          {
             assert( cflags[i].test( ColFlag::kLbInf ) || new_lb > lbs[i] );
 
